# SPDX-License-Identifier: Apache-2.0

DEFAULT_TIMEOUT_MS = 10000
DEFAULT_NAMESPACE_NAME = 'default'
DEFAULT_MARQUEZ_URL = 'http://localhost:8080'
DEFAULT_LIMIT = 100
DEFAULT_OFFSET = 0

API_PATH_V1 = '/api/v1'
